/* eslint-env browser */
import React, {
  MutableRefObject,
  useContext,
  useEffect,
  useMemo,
  useRef,
  useState,
} from 'react';
import {StyleSheet, TouchableOpacity} from 'react-native';
// import type {DateObject} from 'react-native-calendars';
import type {GestureResponderEvent} from 'react-native';
import Flex from '../Flex/Flex';
import SvgCalendar from '../icons/SvgCalendar';
import Text from '../Text/Text';
import {BORDER_COLOR, GRAY_95, WHITE} from '../UikitUtils/colors';
import {isValidDate} from '../UikitUtils/helpers';
import CalendarList from './CalendarList';
import Month from './Month';
import Year from './Year';
import {isEmpty} from '../UikitUtils/validators';
import ThemeContext from '../ThemeContext/ThemeContext';
import {useClickOutsideWeb} from './ClickOutsideHook';

type ViewStyle = import('react-native').ViewStyle;
type Styles = {
  border: ViewStyle;
  time: ViewStyle;
  calendar: ViewStyle;
  content: ViewStyle;
  relative: ViewStyle;
  disabled: ViewStyle;
};
const styles = StyleSheet.create<Styles>({
  border: {
    borderWidth: 1,
    backgroundColor: WHITE,
    borderColor: BORDER_COLOR,
    alignItems: 'stretch',
    paddingHorizontal: 12,
    width: '100%',
    paddingVertical: 10,
    borderRadius: 4,
    height: 40,
  },
  time: {
    padding: 16,
  },
  calendar: {
    borderWidth: 1,
    borderColor: BORDER_COLOR,
    borderRadius: 4,
  },
  content: {
    paddingHorizontal: 32,
    paddingBottom: 6,
  },
  relative: {
    position: 'relative',
  },
  disabled: {
    backgroundColor: GRAY_95,
  },
});

type PartialDateObj = {
  day: number;
  month: number;
  year: number;
};

export type InititalDateType = any | undefined;

export type CommonProps = {
  onChange: (arg: string) => void;
  minDate?: string;
  scrollable?: boolean;
};

export type CommonDefaultProps = {
  align: 'bottomLeft' | 'topLeft' | 'topRight';
};

export type DefaultProps = {
  maxDate: string;
};

const defaultProps: DefaultProps & CommonDefaultProps = {
  maxDate: '9999-12-31',
  align: 'topLeft',
};

type Props = {
  selectedDateColor?: string;
  showTime?: boolean;
  value?: string;
  initialHour?: string;
  initialMinute?: string;
  overrideStyle?: ViewStyle;
  placeholder?: string;
  disabled?: boolean;
} & CommonProps &
  DefaultProps &
  CommonDefaultProps;

const CURRENT_DATE = new Date();

export const getDay = (day: number) => (day < 10 ? `0${day}` : `${day}`);
export const getMonth = (month: number) =>
  month < 10 ? `0${month}` : `${month}`;

export const getRequiredPartialDateObj = (date: InititalDateType) => {
  if (date) {
    return {
      day: date.day,
      month: date.month,
      year: date.year,
    };
  }
  return {
    day: CURRENT_DATE.getDate(),
    month: CURRENT_DATE.getMonth() + 1,
    year: CURRENT_DATE.getFullYear(),
  };
};

const getReadableDate = (date: InititalDateType): string => {
  if (!date) return '';
  const day = getDay(date.day);
  const month = getMonth(date.month);
  return `${day}/${month}/${date.year}`;
};

export const getDateString = (date: PartialDateObj): string =>
  `${date.year}-${getMonth(date.month)}-${getDay(date.day)}`;

export const handleCurrentDate = (date: Date): any => {
  const day = date.getDate();
  const month = date.getMonth() + 1;
  const year = date.getFullYear();
  const timestamp = Date.now();
  const dateString = getDateString({day, month, year});
  return {
    day,
    month,
    year,
    timestamp,
    dateString,
  };
};

export const getAbsoluteTime = (time: number): string => {
  let result: string = '';
  if (time < 10) {
    result = `0${time}`;
  } else {
    result = `${time}`;
  }
  return result;
};

export const convertDateStrtoDateObject = (
  datestring: string | undefined,
): InititalDateType => {
  if (!datestring) {
    return undefined;
  }

  if (!isValidDate(datestring)) {
    return undefined;
  }
  const parsedDate = new Date(datestring);
  return handleCurrentDate(parsedDate);
};

const Calendar = ({
  selectedDateColor = 'red',
  showTime = false,
  value,
  initialHour = '',
  initialMinute = '',
  onChange,
  maxDate,
  overrideStyle,
  placeholder,
  disabled,
  minDate,
  align,
  scrollable,
}: Props) => {
  const [open, setOpen] = useState({
    date: false,
    month: false,
    year: false,
  });

  useEffect(() => {
    const requiredDate = convertDateStrtoDateObject(value);
    setDate(requiredDate);
  }, [value]);

  const initialDate = convertDateStrtoDateObject(value);
  const [date, setDate] = useState<InititalDateType>(initialDate);
  const [hour, setHour] = useState(initialHour);
  const [min, setMin] = useState(initialMinute);
  const Ref: MutableRefObject<any> = useRef();
  const themeContext = useContext(ThemeContext);
  const parentClicked = themeContext.parentClicked
    ? themeContext.parentClicked
    : false;

  useEffect(() => {
    if (parentClicked && open) {
      setOpen({
        date: false,
        month: false,
        year: false,
      });
    }
  }, [parentClicked]);

  const handleClickOutside = (e: Event) => {
    if (Ref.current && !Ref.current.contains(e.target)) {
      handleDateClose();
    }
  };

  useClickOutsideWeb({
    handleClick: handleClickOutside,
  });

  const dateInputStyle = useMemo(() => {
    const styleArray = [styles.border];
    if (overrideStyle) {
      styleArray.push(overrideStyle);
    }
    if (disabled) {
      styleArray.push(styles.disabled);
    }
    return StyleSheet.flatten(styleArray);
  }, [disabled]);

  const handleDateOpen = (e: GestureResponderEvent) => {
    if (e) {
      e.stopPropagation();
    }
    if (themeContext.setParentClicked) {
      themeContext.setParentClicked(false);
    }
    setOpen({
      date: true,
      month: false,
      year: false,
    });
  };

  const handleDateClose = () =>
    setOpen({
      date: false,
      month: false,
      year: false,
    });

  const handleOpenMonth = (e: GestureResponderEvent) => {
    e.stopPropagation();
    if (themeContext.setParentClicked) {
      themeContext.setParentClicked(false);
    }
    setOpen({
      date: false,
      month: true,
      year: false,
    });
  };

  const handleCloseMonth = (e: GestureResponderEvent) => {
    e.stopPropagation();
    setOpen({
      date: true,
      month: false,
      year: false,
    });
  };

  const handleSelectMonth = (e: GestureResponderEvent, month: number) => {
    e.stopPropagation();
    const requiredMonth = month + 1;
    const requiredObject = getRequiredPartialDateObj(date);
    const dateString = getDateString({
      ...requiredObject,
      month: requiredMonth,
    });
    const dateObject = convertDateStrtoDateObject(dateString);
    setDate(dateObject);
    handleCloseMonth(e);
  };

  const handleOpenYear = (e: GestureResponderEvent) => {
    e.stopPropagation();
    setOpen({
      date: false,
      month: false,
      year: true,
    });
  };

  const handleCloseYear = (e: GestureResponderEvent) => {
    handleOpenMonth(e);
  };

  let HOUR = '';
  let meridiem = '';
  if (hour) {
    let parsedHour = parseInt(hour, 10);
    if (parsedHour === 12) {
      HOUR = ` | ${getAbsoluteTime(parsedHour)}`;
      meridiem = 'pm';
    } else if (parsedHour > 12) {
      parsedHour -= 12;
      HOUR = ` | ${getAbsoluteTime(parsedHour)}`;
      meridiem = 'pm';
    } else {
      HOUR = ` | ${getAbsoluteTime(parsedHour)}`;
      meridiem = 'am';
    }
  }
  const MIN = min ? `: ${min}` : '';
  const formattedDate = getReadableDate(date);

  const memoizedValue = useMemo(() => formattedDate + HOUR + MIN + meridiem, [
    formattedDate,
    HOUR,
    MIN,
    meridiem,
  ]);
  const overallStyle = useMemo(() => {
    if (open.date || open.month || open.year) {
      return {zIndex: 1};
    }
    return {};
  }, [open.date, open.month, open.year]);

  return (
    <Flex overrideStyle={overallStyle}>
      <TouchableOpacity
        testID={`calendarTestId_${placeholder}`}
        activeOpacity={1}
        ref={Ref}
        onPress={handleDateOpen}
        disabled={disabled}
        style={styles.relative}
      >
        <Flex
          testID="dateInputContainer"
          center
          row
          overrideStyle={dateInputStyle}
        >
          <Flex testID="datePlaceHolder" flex={1} start end>
            {!isEmpty(memoizedValue) ||
              (!isEmpty(placeholder) && (
                <Text testID="placeHolderText" size={14} color="gray">
                  {placeholder}
                </Text>
              ))}
            <Text size={14}>{memoizedValue}</Text>
          </Flex>
          <Flex middle end overrideStyle={{paddingLeft: 10}}>
            <SvgCalendar />
          </Flex>
        </Flex>
      </TouchableOpacity>
      {open.date && (
        <CalendarList
          ref={Ref}
          selectedDateColor={selectedDateColor}
          showTime={showTime}
          chosenDate={date}
          hour={hour}
          min={min}
          setDate={setDate}
          setHour={setHour}
          setMin={setMin}
          onChange={onChange}
          handleClose={handleDateClose}
          handleOpenMonth={handleOpenMonth}
          handleOpenYear={handleOpenYear}
          maxDate={maxDate}
          minDate={minDate}
          align={align}
          scrollable={scrollable}
        />
      )}
      {open.month && (
        <Month ref={Ref} handleSelectMonth={handleSelectMonth} align={align} />
      )}
      {open.year && (
        <Year
          ref={Ref}
          date={date}
          setDate={setDate}
          handleCloseYear={handleCloseYear}
          align={align}
        />
      )}
    </Flex>
  );
};

Calendar.defaultProps = defaultProps;

export default Calendar;
