import React, { useState, useEffect, useMemo } from 'react';
import { Calendar as Calendars } from 'react-native-calendars';
import {
  GestureResponderEvent,
  View,
  ScrollView,
  StyleSheet,
  TouchableOpacity,
} from 'react-native';

import Flex from '../Flex/Flex';
import SvgLeftAngle from '../icons/SvgLeftAngle';
import SvgRightAngle from '../icons/SvgRightAngle';
import Text, { textColors } from '../Text/Text';
import { BORDER_COLOR, WHITE } from '../UikitUtils/colors';
import { ZINDEX_MODAL } from '../UikitUtils/constants';
import CalendarHeader from './CalendarHeader';
import {
  DefaultProps as DefaultPropsTypes,
  CommonProps,
  CommonDefaultProps,
  InititalDateType,
  getAbsoluteTime,
} from './Calendar';

import { commonCalendarStyle } from './CalendarStyle';

const styles = StyleSheet.create({
  select: {
    borderRadius: 100,
    padding: 6,
    width: 32,
  },
  time: {
    padding: 16,
  },
  calendar: {
    borderWidth: 1,
    borderColor: BORDER_COLOR,
    borderRadius: 4,
  },
  content: {
    paddingHorizontal: 32,
    paddingBottom: 6,
    backgroundColor: WHITE,
  },
  innerContainer: {
    position: 'absolute',
    elevation: 10,
    width: '100%',
  },

  scrollable: {
    overflow: 'scroll',
  },
  containerOpen: {
    zIndex: ZINDEX_MODAL,
  },
});
type Props = {
  selectedDateColor: string;
  showTime: boolean;
  chosenDate: InititalDateType;
  setDate: (arg: InititalDateType) => void;
  hour: string;
  setHour: (arg: any) => void;
  min: string;
  setMin: (arg: any) => void;
  handleOpenMonth: (e: GestureResponderEvent) => void;
  handleOpenYear: (e: GestureResponderEvent) => void;
  handleClose: () => void;
} & CommonProps &
  CommonDefaultProps &
  DefaultPropsTypes;

const renderArrow = (
  direction: 'left' | 'right',
  selectedDateColor: string,
) => {
  if (direction === 'right') {
    return <SvgRightAngle fill={selectedDateColor} />;
  }
  if (direction === 'left') {
    return <SvgLeftAngle fill={selectedDateColor} />;
  }
  return <></>;
};

const CURRENT_HOUR = new Date().getHours();
const CURRENT_MIN = new Date().getMinutes();

const CalendarList = (
  {
    selectedDateColor,
    showTime,
    chosenDate,
    setDate,
    hour,
    setHour,
    min,
    setMin,
    onChange,
    handleOpenMonth,
    handleOpenYear,
    handleClose,
    maxDate,
    minDate,
    align,
    scrollable,
  }: Props,
  ref: React.LegacyRef<View> | undefined,
) => {
  const dateStr = chosenDate ? chosenDate.dateString : '';
  const [markedDate, setMarkedDate] = useState(dateStr);
  const innerContainerStyle = useMemo(() => {
    const styleContainer = [];
    switch (align) {
      case 'topRight':
        styleContainer.push(styles.innerContainer);
        styleContainer.push(commonCalendarStyle.topRightContainer);
        break;
      case 'bottomLeft':
        styleContainer.push(styles.innerContainer);
        styleContainer.push(commonCalendarStyle.bottomContainer);
        break;
      default:
        styleContainer.push(styles.innerContainer);
        break;
    }

    if (scrollable) {
      styleContainer.push(styles.scrollable);
    }
    return StyleSheet.flatten(styleContainer);
  }, [align, scrollable]);

  useEffect(() => {
    if (dateStr) {
      setMarkedDate(dateStr);
    }
  }, [dateStr]);

  const HOURS = [];
  for (let i = 0; i < 24; i++) {
    if (i < 10) HOURS.push(`0${i}`);
    else HOURS.push(`${i}`);
  }
  const MINUTES = [];
  for (let i = 0; i < 60; i++) {
    if (i < 10) MINUTES.push(`0${i}`);
    else MINUTES.push(`${i}`);
  }

  const getRequiredDateString = (day: string, hr: string, minutes: string) => {
    let result = day;
    if (showTime) {
      if (hr === '' && minutes === '') {
        result += ' 00:00:00';
      } else if (hr !== '' && minutes === '') {
        result += ` ${hr}:00:00`;
      } else if (hr === '' && minutes !== '') {
        result += ` 00:${minutes}:00`;
      } else {
        result += ` ${hr}:${minutes}:00`;
      }
    }
    return result;
  };

  const handleDate = (day: any) => {
    setMarkedDate(day.dateString);
    setDate(day);
    const reqDate = getRequiredDateString(day.dateString, hour, min);
    onChange(reqDate);
    handleClose();
  };

  const handleColor = (
    item: string,
    index: number,
    time: number,
    selectedTime: number,
  ) => {
    let COLOR: textColors = 'primary';
    if (parseInt(item, 10) === time) {
      COLOR = 'link';
    }
    if (selectedTime === index) {
      COLOR = 'white';
    }
    if (parseInt(item, 10) === time && selectedTime === index) {
      COLOR = 'white';
    }
    return COLOR;
  };

  const handleHour = (cur_hour: number) => {
    const result = getAbsoluteTime(cur_hour);
    setHour(result);
    if (!min) {
      setMin('00');
    }
    const reqDate = getRequiredDateString(dateStr, result, min);
    onChange(reqDate);
  };
  const handleMinute = (cur_min: number) => {
    const result = getAbsoluteTime(cur_min);
    setMin(result);
    if (!hour) {
      setHour('00');
    }
    const reqDate = getRequiredDateString(dateStr, hour, result);
    onChange(reqDate);
  };

  const parsedHour = parseInt(hour, 10);
  const parsedMin = parseInt(min, 10);
  return (
    <View ref={ref} testID="calendarList">
      <Flex
        row
        overrideStyle={innerContainerStyle}
        testID="calendarListContainer"
      >
        <>
          <Calendars
            maxDate={maxDate}
            minDate={minDate}
            current={markedDate}
            onDayPress={handleDate}
            enableSwipeMonths
            renderArrow={(direction) =>
              renderArrow(direction, selectedDateColor)}
            renderHeader={(currentDate) => (
              <CalendarHeader
                date={currentDate}
                handleOpenMonth={handleOpenMonth}
                handleOpenYear={handleOpenYear}
              />
            )}
            markedDates={{
              [markedDate]: {
                selected: true,
                selectedColor: selectedDateColor,
              },
            }}
            style={styles.calendar}
          />
        </>
        {showTime && (
          <Flex around row overrideStyle={[styles.calendar, styles.content]}>
            <Flex overrideStyle={{ height: 346 }}>
              <Text overrideStyle={styles.time} align='center' bold>
                Hour
              </Text>
              <ScrollView showsHorizontalScrollIndicator={false}>
                {HOURS.map((item, index) => {
                  const color = handleColor(
                    item,
                    index,
                    CURRENT_HOUR,
                    parsedHour,
                  );
                  return (
                    <Flex key={item} center>
                      <TouchableOpacity
                        style={[
                          styles.select,
                          {
                            backgroundColor:
                              parsedHour === index ? selectedDateColor : '',
                          },
                        ]}
                        onPress={() => handleHour(index)}
                      >
                        <Text color={color} size={16} align='center'>
                          {item}
                        </Text>
                      </TouchableOpacity>
                    </Flex>
                  );
                })}
              </ScrollView>
            </Flex>
            <Flex overrideStyle={{ height: 346 }}>
              <Text overrideStyle={styles.time} align='center' bold>
                Min
              </Text>
              <ScrollView showsHorizontalScrollIndicator={false}>
                {MINUTES.map((item, index) => {
                  const color = handleColor(
                    item,
                    index,
                    CURRENT_MIN,
                    parsedMin,
                  );
                  return (
                    <Flex key={item} center>
                      <TouchableOpacity
                        style={[
                          styles.select,
                          {
                            backgroundColor:
                              parsedMin === index ? selectedDateColor : '',
                          },
                        ]}
                        onPress={() => handleMinute(index)}
                      >
                        <Text color={color} size={16} align='center'>
                          {item}
                        </Text>
                      </TouchableOpacity>
                    </Flex>
                  );
                })}
              </ScrollView>
            </Flex>
          </Flex>
        )}
      </Flex>
    </View>
  );
};

export default React.forwardRef(CalendarList);
