import React, {memo, useCallback, useContext} from 'react';
import {TouchableOpacity, StyleSheet} from 'react-native';
import SvgCheckBox from '../icons/SvgCheckBox';
import SvgCheckBoxOutline from '../icons/SvgCheckBoxOutline';
import {textColors} from '../Text/Text';
import ThemeContext from '../ThemeContext/ThemeContext';
import {primaryColor} from '../UikitUtils/themeHelper';
import {isEmpty} from '../UikitUtils/validators';
import CheckboxLabel from './CheckboxLabel';

type DefaultPropsTypes = {
  name: string;
  size: number;
};

const defaultProps: DefaultPropsTypes = {
  size: 24,
  name: '',
};

export type CheckBoxEventType = {
  target: {
    value: any[];
  };
};

type Props = {
  onClick?: (args: any) => void;
  onBlur?: (args: any) => void;
  checked?: boolean;
  size: number;
  color?: string;
  label?: import('react').ReactNode;
  value?: any[];
  labelColor?: textColors;
  labelSize?: number;
  testID?: string;
} & typeof defaultProps;

const styles = StyleSheet.create({
  container: {
    alignItems: 'center',
    flexDirection: 'row',
  },
});

const CheckBox = ({
  color,
  size,
  onClick,
  checked,
  onBlur,
  label,
  labelColor,
  labelSize,
  name,
  value,
  testID,
}: Props) => {
  const {theme} = useContext(ThemeContext);

  const handleOnClick = useCallback(
    (e: any) => {
      const isValuePresent = Array.isArray(value);
      if (typeof onClick === 'function') {
        if (!value) {
          e.target.value = [name];
        } else if (isValuePresent) {
          if (value.includes(name)) {
            const filteredValue = value.filter(v => v !== name);
            e.target.value = filteredValue;
          } else {
            e.target.value = [...value, name];
          }
        }
        onClick(e);
      }
    },
    [onClick],
  );

  let boxColor;
  if (isEmpty(color)) {
    boxColor = primaryColor(theme);
  } else {
    boxColor = color;
  }
  return (
    <TouchableOpacity
      testID={testID}
      onPress={handleOnClick}
      onBlur={onBlur}
      activeOpacity={1}
      style={styles.container}
    >
      {checked ? (
        <SvgCheckBox width={size} height={size} fill={boxColor} />
      ) : (
        <SvgCheckBoxOutline width={size} height={size} />
      )}
      <CheckboxLabel
        label={label}
        labelColor={labelColor}
        labelSize={labelSize}
      />
    </TouchableOpacity>
  );
};

CheckBox.defaultProps = defaultProps;

export default memo(CheckBox);
