import React, {useState, memo, useEffect} from 'react';
import {
  Image as ImageComponent,
  Dimensions,
  ImageStyle,
  View,
  StyleSheet,
  TouchableOpacity,
} from 'react-native';
import type {ImageResizeMode} from 'react-native';
import {isEmpty, secureImage} from '../UikitUtils/validators';
import {GRAY_90, WHITE, GRAY_70} from '../UikitUtils/colors';
import Modal from '../Modal/Modal';
import Flex from '../Flex/Flex';
import Button from '../Button/Button';
import SvgClose from '../icons/SvgClose';

const {width: windowWidth} = Dimensions.get('window');

const styles = StyleSheet.create({
  overAll: {
    backgroundColor: WHITE,
    borderRadius: 10,
    position: 'relative',
  },
  svgClose: {
    position: 'absolute',
    zIndex: 11,
    top: 16,
    right: 16,
  },
  imagePreviewSize: {
    height: '80%',
    width: '80%',
  },
});

const defaultProps = {
  size: 250,
  overrideStyle: {},
  previewMode: false,
};

type ResponsiveProps = {
  responsive?: boolean;
};

type sizeProp = {
  size: number;
};

type Props = {
  src: string;
  alt?: string;
  resizeMode?: ImageResizeMode;
  height?: number | string;
  width?: number | string;
  testID?: string;
  headers?: {
    [key: string]: string;
  };
} & typeof defaultProps &
  ResponsiveProps;

export const onSuccessLoad = (
  imageWidth: number,
  imageHeight: number,
  props: ResponsiveProps & sizeProp,
  setWidth: (arg: number) => void,
  setHeight: (arg: number) => void,
  setSizeMode: (arg: ImageResizeMode) => void,
) => {
  const {responsive, size} = props;
  if (!responsive) {
    return;
  }
  if (imageWidth < size) {
    setWidth(imageWidth);
  } else if (imageWidth < windowWidth) {
    setWidth(imageWidth * 0.4);
    setSizeMode('contain');
  } else {
    setWidth(windowWidth * 0.8);
    setSizeMode('contain');
  }

  if (imageHeight < size) {
    setHeight(imageHeight);
  } else {
    setHeight(imageHeight * 0.4);
    setSizeMode('contain');
  }
};

const Image = ({
  src,
  overrideStyle,
  resizeMode,
  responsive,
  size,
  alt,
  height,
  width,
  testID,
  previewMode,
  headers,
}: Props) => {
  const [isOpenPreview, setOpenPreview] = useState(false);

  const imgHeight = responsive ? size : height;
  const imgWidth = responsive ? size : width;
  const [currentHeight, setHeight] = useState(imgHeight);
  const [currentWidth, setWidth] = useState(imgWidth);
  const [showImagePlaceholder, setShowImagePlaceholder] = useState(false);
  const [imageContainerStyle, setImageContainerStyle] = useState<ImageStyle[]>(
    [],
  );
  let requiredResizeMode: ImageResizeMode | undefined;
  if (!isEmpty(resizeMode)) {
    requiredResizeMode = resizeMode;
  }
  if (responsive) {
    requiredResizeMode = 'cover';
  }
  const [sizeMode, setSizeMode] = useState(requiredResizeMode);
  useEffect(() => {
    setImageContainerStyle([
      {height: currentHeight, width: currentWidth},
      overrideStyle,
    ]);
  }, [currentHeight, currentWidth]);

  const accessibilityLabel = alt || src;

  const failure = () => {
    setShowImagePlaceholder(true);
  };

  if (src && typeof ImageComponent.getSize === 'function') {
    ImageComponent.getSize(
      src,
      (imageWidth, imageHeight) =>
        onSuccessLoad(
          imageWidth,
          imageHeight,
          {responsive, size},
          setWidth,
          setHeight,
          setSizeMode,
        ),
      failure,
    );
  }

  const handlePreview = (value: boolean) => {
    if (value) {
      setOpenPreview(true);
    } else {
      setOpenPreview(false);
    }
  };

  if (showImagePlaceholder) {
    return (
      <View
        style={[imageContainerStyle, {backgroundColor: GRAY_90}]}
        testID="image-placeholder"
      />
    );
  }

  if (isOpenPreview) {
    return (
      <Modal animationType='slide' open={isOpenPreview}>
        <Flex center middle overrideStyle={styles.overAll}>
          <Button
            onClick={() => handlePreview(false)}
            type="link"
            overrideStyle={styles.svgClose}
          >
            <SvgClose fill={GRAY_70} />
          </Button>
          <ImageComponent
            style={styles.imagePreviewSize}
            source={{uri: secureImage(src)}}
            resizeMode={sizeMode}
            accessibilityLabel={accessibilityLabel}
            accessible
          />
        </Flex>
      </Modal>
    );
  }

  return (
    <TouchableOpacity
      disabled={!previewMode}
      onPress={() => handlePreview(true)}
    >
      <ImageComponent
        accessible
        source={{uri: secureImage(src), headers}}
        style={imageContainerStyle}
        resizeMode={sizeMode}
        accessibilityLabel={accessibilityLabel}
        testID={testID}
      />
    </TouchableOpacity>
  );
};

Image.defaultProps = defaultProps;

export default memo(
  Image,
  (prevProps, nextProps) => prevProps.src === nextProps.src,
);
