import React, {createRef, useContext, useEffect, useRef, useState} from 'react';
import {
  KeyboardTypeOptions,
  NativeSyntheticEvent,
  StyleSheet,
  TextInput,
  TextInputKeyPressEventData,
  TouchableOpacity,
  View,
  ViewStyle,
} from 'react-native';
import SvgCloseSmall from '../icons/SvgCloseSmall';
import Flex from '../Flex/Flex';
import {textColors} from '../Text/Text';
import {textColorHelper} from '../Text/TextHelper';
import ThemeContext from '../ThemeContext/ThemeContext';
import {GRAY_70} from '../UikitUtils/colors';
import {primaryColor} from '../UikitUtils/themeHelper';
import Action from './Action';
import {styles} from './InputStyle';
import {
  inputAlginHelper,
  inputActionHelper,
  inputTypesHelper,
  inputHelper,
  textStyleHelper,
  minHeights,
} from './InputTextHelper';
import LabelWrapper from './LabelWrapper';
import ErrorMessage from '../ErrorMessage/ErrorMessage';
import {isEmpty} from '../UikitUtils/validators';

export type InputAlign = 'left' | 'right' | 'center' | 'top';
export type InputTypes = 'normal' | 'curved' | 'rounded' | 'noBorder';
export type returnKeyType =
  | 'default'
  | 'done'
  | 'none'
  | 'search'
  | 'go'
  | 'next'
  | 'send'
  | 'previous'
  | 'google'
  | 'join'
  | 'route'
  | 'yahoo'
  | 'emergency-call'
  | undefined;

export type textContentType =
  | 'none'
  | 'URL'
  | 'addressCity'
  | 'addressCityAndState'
  | 'addressState'
  | 'countryName'
  | 'creditCardNumber'
  | 'emailAddress'
  | 'familyName'
  | 'fullStreetAddress'
  | 'givenName'
  | 'jobTitle'
  | 'location'
  | 'middleName'
  | 'name'
  | 'namePrefix'
  | 'nameSuffix'
  | 'nickname'
  | 'organizationName'
  | 'postalCode'
  | 'streetAddressLine1'
  | 'streetAddressLine2'
  | 'sublocality'
  | 'telephoneNumber'
  | 'username'
  | 'password'
  | 'newPassword'
  | 'oneTimeCode'
  | undefined;

export type autoCapitalize =
  | 'none'
  | 'sentences'
  | 'words'
  | 'characters'
  | undefined;

export const isInputTextTypeKey = (
  str: string | undefined,
): str is InputTypes => str !== undefined;

type DefaultProps = {
  type: InputTypes;
  align: InputAlign;
  size: number;
  value: string;
  placeholderTextColor: string;
  minHeight: number;
  color: textColors;
  allowFontScaling: boolean;
  name: string;
  formikErrors: {[key: string]: string | undefined};
  formikTouched: {[key: string]: boolean | undefined};
  error: boolean | any;
};
const defaultProps: DefaultProps = {
  type: 'curved',
  align: 'left',
  size: 16,
  value: '',
  placeholderTextColor: GRAY_70,
  minHeight: 44,
  color: 'primary',
  allowFontScaling: false,
  name: '',
  formikErrors: {'': ''},
  formikTouched: {'': false},
  error: false,
};

type Props = {
  required?: boolean;
  label?: string;
  autoFocus?: boolean;
  placeholder?: string;
  onChange?: (text: string) => void;
  onChangeNative?: (arg: any) => any;
  actionLeft?: Function;
  actionRight?: Function;
  flex?: number;
  overrideStyle?: ViewStyle;
  onBlur?: (arg: any) => any;
  onFocus?: (arg: any) => void;
  onClear?: boolean;
  disabled?: boolean;
  bold?: boolean;
  size?: number;
  multiline?: boolean;
  numberOfLines?: number;
  getRef?: (ref: React.RefObject<TextInput>) => void;
  onKeyPress?:
    | ((e: NativeSyntheticEvent<TextInputKeyPressEventData>) => void)
    | undefined;
  onSubmitEditing?: () => void;
  returnKeyType?: returnKeyType;
  secureTextEntry?: boolean;
  maxLength?: number;
  keyboardType?: KeyboardTypeOptions;
  textContentType?: textContentType;
  autoCapitalize?: autoCapitalize;
  white?: boolean;
  labelSize?: number;
  labelColor?: textColors;
  underlineColorAndroid?:string
} & typeof defaultProps;

const InputText = ({
  overrideStyle,
  onBlur,
  onFocus,
  required,
  label,
  type,
  autoFocus,
  value,
  placeholder,
  onChange,
  onChangeNative,
  actionLeft,
  actionRight,
  color,
  align,
  error,
  flex,
  disabled,
  bold,
  size,
  multiline,
  numberOfLines,
  onClear,
  onKeyPress,
  keyboardType,
  onSubmitEditing,
  returnKeyType,
  maxLength,
  secureTextEntry,
  placeholderTextColor,
  minHeight,
  getRef,
  textContentType,
  autoCapitalize,
  white,
  allowFontScaling,
  labelSize,
  labelColor,
  name,
  formikErrors,
  formikTouched,
  underlineColorAndroid
}: Props) => {
  const {theme} = useContext(ThemeContext);
  const ref = useRef(null);
  const textInput = createRef<TextInput>();
  const [containerStyle, setContainerStyle] = useState<ViewStyle[]>([]);
  const [textStyle, setTextStyle] = useState<ViewStyle[]>([]);
  const [actionLeftStyle, setActionLeftStyle] = useState<ViewStyle[]>([]);
  const [actionRightStyle, setActionRightStyle] = useState<ViewStyle[]>([]);
  const [isFocused, setIsFocused] = useState<boolean>(false);

  const removeOutline: any = {outlineStyle: 'none'};

  useEffect(() => {
    handleContainerStyle();
  }, [error, disabled, overrideStyle]);

  useEffect(() => {
    if (typeof getRef === 'function' && textInput && textInput.current) {
      getRef(textInput);
    }
  }, []);

  const handleContainerStyle = () => {
    const containerStyleArray = [styles.relativestyle];
    const textStyleArray = [removeOutline];
    const actionLeftStyleArray: ViewStyle[] = [];
    const actionRightStyleArray: ViewStyle[] = [];

    inputTypesHelper({theme, type, containerStyleArray});
    setContainerStyle(containerStyleArray);
    setTextStyle(textStyleArray);

    inputActionHelper({
      actionLeftStyleArray,
      actionRightStyleArray,
      type,
      theme,
    });
    inputAlginHelper(align, textStyleArray);
    inputHelper({
      theme,
      containerStyleArray,
      flex,
      error,
      overrideStyle,
      disabled,
      white,
    });
    textStyleHelper({
      theme,
      actionRight,
      actionLeft,
      textStyleArray,
      bold,
      size,
      disabled,
      type,
    });
    minHeights(minHeight, textStyleArray);

    const requiredColor = textColorHelper(theme, color);
    const textColorStyle = {color: requiredColor} as ViewStyle;
    textStyleArray.push(textColorStyle);
    setTextStyle(textStyleArray);
    setActionLeftStyle(actionLeftStyleArray);
    setActionRightStyle(actionRightStyleArray);
  };

  if (error && isFocused) {
    setIsFocused(false);
  }
  const handleClear = () => {
    if (onClear && typeof onChange === 'function') {
      onChange('');
    }
  };
  const handleFocus = (e: any) => {
    if (!error) {
      setIsFocused(true);
    }
    if (typeof onFocus === 'function') {
      onFocus(e);
    }
  };

  const handleBlur = (e: any) => {
    setIsFocused(false);
    if (typeof onBlur === 'function') {
      onBlur(e);
    }
  };

  const inputTextStyleObject = StyleSheet.flatten(textStyle);

  const focussedStyle =
    isFocused && !disabled ? {borderColor: primaryColor(theme)} : {};

  return (
    <>
      <LabelWrapper
        required={required}
        label={label}
        labelColor={labelColor}
        labelSize={labelSize}
        error={error}
      >
        <View ref={ref} style={[...containerStyle, focussedStyle]}>
          <View style={styles.containerActionStyle}>
            {typeof actionLeft === 'function' && (
              <Action action={actionLeft} actionStyle={actionLeftStyle} />
            )}
            <TextInput
              allowFontScaling={allowFontScaling}
              textContentType={textContentType}
              autoCapitalize={autoCapitalize}
              ref={textInput}
              keyboardType={keyboardType}
              multiline={multiline}
              style={inputTextStyleObject}
              onChangeText={onChange}
              onChange={onChangeNative}
              placeholder={placeholder}
              autoFocus={autoFocus}
              value={value}
              editable={!disabled}
              numberOfLines={numberOfLines}
              onBlur={handleBlur}
              onFocus={handleFocus}
              onKeyPress={onKeyPress}
              onSubmitEditing={onSubmitEditing}
              returnKeyType={returnKeyType}
              secureTextEntry={secureTextEntry}
              maxLength={maxLength}
              placeholderTextColor={placeholderTextColor}
              underlineColorAndroid={underlineColorAndroid}
            />
            {onClear && !disabled && (
              <Flex overrideStyle={styles.enableClose}>
                <TouchableOpacity onPress={handleClear}>
                  <SvgCloseSmall />
                </TouchableOpacity>
              </Flex>
            )}
            {typeof actionRight === 'function' && (
              <Action action={actionRight} actionStyle={actionRightStyle} />
            )}
          </View>
        </View>
      </LabelWrapper>
      {!isEmpty(error) && (
        <View style={styles.errorView}>
          <ErrorMessage
            name={name}
            touched={formikTouched}
            errors={formikErrors}
          />
        </View>
      )}
    </>
  );
};

InputText.defaultProps = defaultProps;

export default InputText;
