import React, {memo, ReactNode, useContext} from 'react';
import {
  StyleSheet,
  Modal as ReactModal,
  Dimensions,
  TouchableWithoutFeedback,
  View,
} from 'react-native';
import Flex from '../Flex/Flex';
import {WHITE, BLACK} from '../UikitUtils/colors';
import {ZINDEX_MODAL} from '../UikitUtils/constants';
import Button from '../Button/Button';
import SvgClose from '../icons/SvgClose';
import ModalIcon from './ModalIcon';
import {ThemeContext} from '../ThemeContext/ThemeContext';
import Loader from '../Loader/Loader';
import {getColors} from '../UikitUtils/themeHelper';

export type animationType = 'none' | 'fade' | 'slide' | undefined;

const {height} = Dimensions.get('window');

const uikitStyles = StyleSheet.create({
  close: {
    paddingTop: 20,
    paddingRight: 20,
    position: 'absolute',
    alignSelf: 'flex-end',
    zIndex: ZINDEX_MODAL,
  },
  contentContainer: {
    backgroundColor: WHITE,
    borderRadius: 4,
    position: 'relative',
  },
  flexContainer: {
    padding: 24,
  },
  flexMessage: {
    paddingTop: 20,
    paddingBottom: 40,
  },
  linkButton: {
    marginRight: 20,
  },
  secondaryButton: {
    marginRight: 20,
  },
  container: {
    justifyContent: 'center',
    alignItems: 'center',
    backgroundColor: 'rgba(0,0,0,0.25)',
    zIndex: 999,
    height,
  },
});

type Props = {
  open: boolean;
  children?: ReactNode;
  onClose?: () => void;
  disable?: boolean;
  title?: string;
  primaryAction?: string;
  primaryActionHandler?: () => void;
  secondaryAction?: string;
  secondaryActionHandler?: () => void;
  showClose?: boolean;
  showIcon?: (height: number, width: number) => ReactNode;
  iconSize?: 'normal' | 'large';
  isLoading?: boolean;
  animationType?: animationType;
};

export const handleUpdatezIndex = (node?: HTMLElement) => {
  const refNode = node;
  const clasnames = refNode?.getAttribute('class');
  if (
    !clasnames?.includes('navChildren') &&
    refNode &&
    refNode.tagName !== 'BODY'
  ) {
    refNode.style.zIndex = 'auto';
    const typedParentElement = refNode.parentNode as HTMLElement;
    handleUpdatezIndex(typedParentElement);
  }
};

const Modal = ({
  open,
  children,
  onClose,
  disable,
  title,
  primaryAction,
  primaryActionHandler,
  secondaryAction,
  secondaryActionHandler,
  showClose,
  showIcon,
  iconSize,
  isLoading,
  animationType,
}: Props) => {
  const {theme} = useContext(ThemeContext);
  const {PRIMARY_COLOR} = getColors(theme);

  return open ? (
    <ReactModal
      animationType={animationType}
      transparent
      visible
      onRequestClose={onClose}
    >
      <TouchableWithoutFeedback disabled={disable} onPress={onClose}>
        <View style={uikitStyles.container}>
          {isLoading && <Loader />}
          {children && !title && <>{children}</>}
          {title && children && (
            <Flex overrideStyle={uikitStyles.contentContainer}>
              {showClose && iconSize !== 'large' && (
                <Flex overrideStyle={uikitStyles.close}>
                  <Button type="link" onClick={onClose} testID="closeModalIcon">
                    <SvgClose fill={BLACK} width={24} height={24} />
                  </Button>
                </Flex>
              )}
              <Flex overrideStyle={uikitStyles.flexContainer}>
                <ModalIcon
                  title={title}
                  showIcon={showIcon}
                  iconSize={iconSize}
                />
                <Flex
                  row
                  overrideStyle={uikitStyles.flexMessage}
                  middle={iconSize === 'large'}
                >
                  {children}
                </Flex>
                <Flex row bottom middle={iconSize === 'large'}>
                  {secondaryAction &&
                    (iconSize === 'large' ? (
                      <Button
                        type="secondary"
                        onClick={secondaryActionHandler}
                        overrideStyle={uikitStyles.secondaryButton}
                      >
                        {secondaryAction}
                      </Button>
                    ) : (
                      !showClose && (
                        <Button
                          type="link"
                          onClick={secondaryActionHandler}
                          overrideStyle={uikitStyles.linkButton}
                        >
                          {secondaryAction}
                        </Button>
                      )
                    ))}
                  {primaryAction && (
                    <Button
                      flat
                      onClick={primaryActionHandler}
                      overrideStyle={StyleSheet.flatten([
                        {backgroundColor: PRIMARY_COLOR},
                      ])}
                    >
                      {primaryAction}
                    </Button>
                  )}
                </Flex>
              </Flex>
            </Flex>
          )}
        </View>
      </TouchableWithoutFeedback>
    </ReactModal>
  ) : null;
};

export default memo(Modal, (prevProps: Props, nextProps: Props) => {
  if (
    nextProps.title &&
    prevProps.title &&
    prevProps.children &&
    nextProps.children
  ) {
    return (
      prevProps.open === nextProps.open &&
      prevProps.title === nextProps.title &&
      prevProps.children === nextProps.children
    );
  }
  return prevProps.children === nextProps.children;
});
