import React, {useEffect, useState} from 'react';
import type {ReactNode} from 'react';
import {FlatList, StyleSheet, TouchableOpacity} from 'react-native';
import type {ViewStyle, TextStyle} from 'react-native';
import SvgSearch from '../icons/SvgSearch';

import Flex from '../Flex/Flex';
import type {textColors} from '../Text/Text';
import {WHITE} from '../UikitUtils/colors';
import InputText from '../InputText/InputText';
import SelectRowList from './SelectRowList';
import {defaultProps as selectProps} from './SelectTag';
import type {
  DefaultPropTypes as selectDefaultProps,
  CommonPropType,
  list,
} from './SelectTag.types';

export type DefaultProps = {
  options: list[];
  size?: number;
  color?: textColors;
  bold?: boolean;
  searchPlaceHolder?: string;
} & selectDefaultProps;

export const defaultProps: DefaultProps = {
  // shd not be added here
  ...selectProps,
  options: [],
  size: 16,
  color: 'primary',
  bold: false,
  searchPlaceHolder: 'Search',
};

type Props = {
  onChange: Function;
  handleClose: Function;
  renderOption?: (a: list) => ReactNode;
  overrideStyleOptionsItem?: TextStyle;
  resultItem: string;
  multiselect?: boolean;
  handleOnSelect?: (arg: string) => void;
  searchListTestId?: string;
} & typeof defaultProps &
  CommonPropType;

const styles = StyleSheet.create({
  optionsContainer: {
    backgroundColor: WHITE,
    maxHeight: 400,
  },
  inputStyle: {
    marginHorizontal: 8,
    marginVertical: 8,
  },
});

const SelectList = ({
  labelKey,
  options,
  onChange,
  handleClose,
  size,
  color,
  bold,
  renderOption,
  valueKey,
  overrideStyleOptionsItem,
  maxHeight,
  resultItem,
  search,
  searchBy,
  multiselect,
  handleOnSelect,
  searchListTestId,
  searchPlaceHolder,
  showsVerticalScrollIndicator,
}: Props) => {
  const [optionContainerStyle, setOptionContainerStyle] = useState<ViewStyle[]>(
    [styles.optionsContainer],
  );

  const [dropdownOptions, setDropdownOptions] = useState<list[]>(options);
  const [searchValue, setSearchValue] = useState('');

  useEffect(() => {
    if (multiselect) {
      let filteredOptions: list[] = [];
      dropdownOptions.forEach(itemInit => {
        const filteredOption = options.filter(
          item => item[valueKey] === itemInit[valueKey],
        );
        filteredOptions = [...filteredOptions, ...filteredOption];
      });
      setDropdownOptions(filteredOptions);
    } else {
      setDropdownOptions(options);
    }
  }, [options]);

  useEffect(() => {
    setOptionContainerStyle([
      ...optionContainerStyle,
      {
        maxHeight,
      },
    ]);
  }, []);

  const filterCallback = (
    key: string,
    option: {[key: string]: any},
    value: string,
  ) =>
    `${option[key]}`.toLowerCase().trim().includes(value.toLowerCase().trim());
  const handleFilterSearch = (value: string) => {
    setSearchValue(value);
    if (value && typeof value === 'string') {
      let filteredResult: list[] = [];
      if (Array.isArray(searchBy)) {
        searchBy.forEach(searchByValue => {
          const filteredResultItem = options.filter(option =>
            filterCallback(searchByValue, option, value),
          );
          filteredResult = [...filteredResult, ...filteredResultItem];
        });
      } else {
        filteredResult = options.filter(option =>
          filterCallback(labelKey, option, value),
        );
      }
      setDropdownOptions(filteredResult);
    } else {
      setDropdownOptions(options);
    }
  };
  return (
    <Flex overrideStyle={optionContainerStyle} testID="SelectList">
      {search && (
        <TouchableOpacity onFocus={e => e.stopPropagation()}>
          <InputText
            value={searchValue}
            onChange={handleFilterSearch}
            overrideStyle={styles.inputStyle}
            placeholder={searchPlaceHolder}
            actionLeft={() => <SvgSearch />}
          />
        </TouchableOpacity>
      )}

      <FlatList
        keyExtractor={item => item[valueKey]}
        extraData={{value: resultItem || ''}}
        showsVerticalScrollIndicator={showsVerticalScrollIndicator}
        showsHorizontalScrollIndicator={false}
        data={dropdownOptions}
        renderItem={({item, index}) => (
          <SelectRowList
            overrideStyleOptionsItem={overrideStyleOptionsItem}
            valueKey={valueKey}
            renderOption={renderOption}
            bold={bold}
            size={size}
            color={color}
            labelKey={labelKey}
            options={dropdownOptions}
            onChange={onChange}
            handleClose={handleClose}
            item={item}
            index={index}
            searchBy={searchBy}
            multiselect={multiselect}
            handleOnSelect={handleOnSelect}
            searchListTestId={searchListTestId}
          />
        )}
      />
    </Flex>
  );
};

SelectList.defaultProps = defaultProps;

export default SelectList;
