/* eslint-env browser, node */
import React, {
  ReactNode,
  useEffect,
  useState,
  useContext,
  memo,
  useMemo,
} from 'react';
import {
  GestureResponderEvent,
  Platform,
  StyleSheet,
  TouchableOpacity,
  View,
} from 'react-native';
import InputText from '../InputText/InputText';
import {ZINDEX_MODAL} from '../UikitUtils/constants';
import SelectList from './SelectList';
import ThemeContext from '../ThemeContext/ThemeContext';
import {
  BORDER_COLOR,
  WHITE,
  GRAY_95,
  SEMANTIC_RED_PRIMARY,
} from '../UikitUtils/colors';
import DropDownButton from './DropDownButton';
import {useComponentVisible} from '../UikitUtils/helpers';
import SelectListAnimation from './SelectListAnimation';
import {
  checkedAppendFunction,
  handleDisplayValue,
  handleSelectedOption,
} from './SelectTagHelper';
import LabelWrapper from '../InputText/LabelWrapper';
import type {
  DefaultPropTypes,
  SelectProps,
  AlignType,
  ViewStyle,
  MultiOptionsEntity,
} from './SelectTag.types';
import RenderMultiSelectInput from './RenderMultiSelectInput';
import {isEmpty} from '../UikitUtils/validators';
import ErrorMessage from '../ErrorMessage/ErrorMessage';

export const defaultProps: DefaultPropTypes = {
  labelKey: 'label',
  options: [],
  valueKey: 'value',
  align: 'bottomLeft',
  maxHeight: 400,
  showsVerticalScrollIndicator: false,
  auto: true,
  name: '',
  formikErrors: {'': ''},
  formikTouched: {'': false},
  error: false,
};

const styles = StyleSheet.create({
  input: {
    zIndex: ZINDEX_MODAL,
    justifyContent: 'center',
  },
  selectList: {
    position: 'absolute',
    zIndex: ZINDEX_MODAL,
    width: '100%',
    borderWidth: 1,
    borderColor: GRAY_95,
    borderRadius: 8,
    borderTopLeftRadius: 0,
    borderTopRightRadius: 0,
    overflow: 'hidden',
    shadowOffset: {
      width: 0,
      height: 0,
    },
    shadowOpacity: 0.16,
    shadowRadius: 16,
  },
  overAll: {
    position: 'relative',
    zIndex: 1,
  },
  bottomLeft: {
    left: 0,
    top: -40,
  },
  bottomRight: {
    right: 0,
    top: -40,
  },
  topLeft: {
    bottom: 80,
    left: 0,
  },
  overAllOne: {
    position: 'relative',
  },
  inputTagEnabled: {
    backgroundColor: WHITE,
    borderStyle: 'solid',
  },
  inputTagDisabled: {
    borderStyle: 'dashed',
    backgroundColor: GRAY_95,
  },
  inputTagContainer: {
    borderWidth: 1,
    borderRadius: 4,
    justifyContent: 'center',
    paddingRight: 8,
  },
  errorView: {
    marginTop: 3,
    zIndex:-1
  },
  error: {
    borderColor: SEMANTIC_RED_PRIMARY,
    backgroundColor: WHITE,
    borderStyle: 'solid',
  },
});

const SelectTag = ({
  labelKey,
  options,
  onChange,
  value: initalValue,
  size,
  color,
  bold,
  inputColor = 'primary',
  renderOption,
  valueKey,
  iconColor,
  placeholder,
  overrideStyle,
  renderInput,
  align,
  overrideStyleOptionsItem,
  maxHeight,
  disabled,
  required,
  label,
  search,
  testID,
  searchBy,
  multiselect,
  searchListTestId,
  searchPlaceHolder,
  showsVerticalScrollIndicator,
  auto,
  labelSize,
  labelColor,
  name,
  formikErrors,
  formikTouched,
  error,
}: SelectProps) => {
  const [selectListStyle, setSelectListStyle] = useState<ViewStyle[]>([
    styles.selectList,
  ]);
  const [selectAlign, setSelectAlign] = useState<AlignType>(align);
  const [selectOptions, setSelectOptions] = useState<MultiOptionsEntity[]>([]);
  const [resultItem, setResultItem] = useState('');
  const selectTagRef = React.useRef<View>(null);

  const {ref, isComponentVisible, setIsComponentVisible} = useComponentVisible(
    false,
  );

  const stringifyLabelKey = `${labelKey}`;
  const stringifyValueKey = `${valueKey}`;
  const themeContext = useContext(ThemeContext);
  const parentClicked = themeContext.parentClicked
    ? themeContext.parentClicked
    : false;
  const value = useMemo(
    () =>
      multiselect && typeof initalValue === 'string'
        ? initalValue.split(',')
        : initalValue,
    [initalValue],
  );
  /**
   * add all your style logic here
   */
  let inputTagStyle;
  if (disabled) {
    inputTagStyle = styles.inputTagDisabled;
  } else {
    inputTagStyle = styles.inputTagEnabled;
  }

  if (error && !disabled) {
    inputTagStyle = styles.error;
  }

  const isCheckedOptions = Array.isArray(options)
    ? useMemo(
        () =>
          checkedAppendFunction({
            options,
            value,
            stringifyValueKey,
          }),
        [options],
      )
    : [];
  const prefillValue = useMemo(
    () => handleDisplayValue(isCheckedOptions, labelKey),
    [isCheckedOptions],
  );

  useEffect(() => {
    setSelectOptions(isCheckedOptions);
    setResultItem(prefillValue.toString());
  }, [options]);

  useEffect(() => {
    if (selectAlign === 'bottomLeft') {
      setSelectListStyle([styles.selectList, styles.bottomLeft]);
    }
    if (selectAlign === 'topLeft') {
      setSelectListStyle([styles.selectList, styles.topLeft]);
    } else {
      setSelectListStyle([styles.selectList, styles.bottomRight]);
    }
  }, [selectAlign]);

  useEffect(() => {
    if (parentClicked && isComponentVisible) {
      setIsComponentVisible(false);
      console.log('parentClicked: ', parentClicked);
    }
  }, [parentClicked]);

  const {
    labelArray: [result],
  } = useMemo(
    () =>
      handleSelectedOption({
        value,
        selectOptions,
        stringifyValueKey,
        stringifyLabelKey,
      }),
    [value, selectOptions],
  );
  const containerStyle = useMemo(() => {
    if (Platform.OS !== 'web') {
      return styles.overAllOne;
    }
    if (Platform.OS === 'web' && isComponentVisible) {
      return styles.overAll;
    }
    return styles.overAllOne;
  }, [isComponentVisible]);

  const handleOpen = (e: GestureResponderEvent) => {
    if (
      typeof Window !== 'undefined' &&
      auto &&
      selectTagRef &&
      selectTagRef.current
    ) {
      selectTagRef.current.measureInWindow((_x, y, _w, h) => {
        if (y + h > window.innerHeight * 0.6) {
          setSelectAlign('topLeft');
        } else {
          setSelectAlign(align);
        }
      });
    } else {
      setSelectAlign('bottomLeft');
    }
    if (e) {
      e.stopPropagation();
    }
    if (themeContext.setParentClicked) {
      themeContext.setParentClicked(false);
    }
    setIsComponentVisible(true);
  };

  const handleClose = () => {
    setIsComponentVisible(false);
  };

  const toggleSelect = (e: GestureResponderEvent) => {
    if (e) {
      e.stopPropagation();
    }
    if (isComponentVisible) {
      handleClose();
    } else {
      handleOpen(e);
    }
  };

  const handleChangeSelectMultiSelect = (val: string) => {
    const {selectedOption, labelArray, valueArray} = handleSelectedOption({
      value: val,
      selectOptions,
      stringifyValueKey,
      stringifyLabelKey,
    });
    setSelectOptions(selectedOption);
    setResultItem(labelArray.toString());
    onChange(valueArray);
  };

  const stringifiedValue = `${value}`;
  return (
    <View style={containerStyle} ref={selectTagRef}>
      <TouchableOpacity
        testID={testID}
        ref={ref}
        disabled={disabled}
        onPress={toggleSelect}
        activeOpacity={1}
      >
        <View style={styles.input}>
          {(() => {
            let resultTag: ReactNode;
            switch (multiselect) {
              case true:
                resultTag = (
                  <LabelWrapper
                    label={label}
                    required={required}
                    labelColor={labelColor}
                    labelSize={labelSize}
                    error={error}
                  >
                    <View
                      style={[
                        styles.inputTagContainer,
                        {
                          borderColor:
                            error && !disabled
                              ? SEMANTIC_RED_PRIMARY
                              : BORDER_COLOR,
                        },
                      ]}
                    >
                      <DropDownButton
                        spining={isComponentVisible}
                        iconColor={iconColor}
                        result={resultItem} // For MultiSelect
                      >
                        <RenderMultiSelectInput
                          displayValue={resultItem}
                          placeholder={placeholder}
                        />
                      </DropDownButton>
                    </View>
                  </LabelWrapper>
                );
                break;
              case undefined:
              case false:
                if (typeof renderInput === 'function') {
                  resultTag = (
                    <DropDownButton
                      spining={isComponentVisible}
                      iconColor={iconColor}
                      result={stringifiedValue} // For MultiSelect
                    >
                      {renderInput()}
                    </DropDownButton>
                  );
                }
                if (typeof renderInput !== 'function') {
                  resultTag = (
                    <InputText
                      error={error}
                      labelColor={labelColor}
                      labelSize={labelSize}
                      label={label}
                      required={required}
                      overrideStyle={StyleSheet.flatten([
                        inputTagStyle,
                        overrideStyle,
                      ])}
                      disabled
                      placeholder={placeholder}
                      color={inputColor}
                      value={result}
                      type="curved"
                      size={size}
                      actionRight={() => (
                        <DropDownButton
                          result={stringifiedValue} // For MultiSelect
                          spining={isComponentVisible}
                          iconColor={iconColor}
                        />
                      )}
                    />
                  );
                }
                break;
            }
            return resultTag;
          })()}
        </View>
      </TouchableOpacity>

      {isComponentVisible && (
        <SelectListAnimation isOpen={isComponentVisible}>
          <View style={selectListStyle} ref={ref} testID="selectListWrapper">
            <SelectList
              searchPlaceHolder={searchPlaceHolder}
              searchListTestId={searchListTestId}
              overrideStyleOptionsItem={overrideStyleOptionsItem}
              valueKey={valueKey}
              renderOption={renderOption}
              bold={bold}
              size={size}
              color={color}
              labelKey={labelKey}
              options={selectOptions}
              onChange={onChange}
              handleClose={handleClose}
              maxHeight={maxHeight}
              resultItem={resultItem}
              search={search}
              searchBy={searchBy}
              multiselect={multiselect}
              handleOnSelect={handleChangeSelectMultiSelect}
              showsVerticalScrollIndicator={showsVerticalScrollIndicator}
            />
          </View>
        </SelectListAnimation>
      )}
      {!isEmpty(error) && (
        <View style={styles.errorView}>
          <ErrorMessage
            name={name}
            touched={formikTouched}
            errors={formikErrors}
          />
        </View>
      )}
    </View>
  );
};

SelectTag.defaultProps = defaultProps;

export default memo(SelectTag);
