import React, {memo, useContext, useState, useEffect} from 'react';
import {StyleSheet, Text as TextComponent} from 'react-native';
import ThemeContext from '../ThemeContext/ThemeContext';
import {textColorHelper, textHelper} from './TextHelper';
import {styles, Styles} from './textStyle';

type TextStyle = import('react-native').TextStyle;
type ViewStyle = import('react-native').ViewStyle;

export type textColors =
  | 'primary'
  | 'error'
  | 'link'
  | 'success'
  | 'white'
  | 'gray'
  | 'warning'
  | 'info'
  | 'black'
  | 'theme';

export type textTransForm = 'capitalize' | 'uppercase' | 'lowercase';
export type textAlignForm = 'left' | 'justify' | 'center' | 'right';

type DefaultProps = {
  size: number;
  color: textColors;
  align: textAlignForm;
  allowFontScaling: boolean;
};
const defaultProps: DefaultProps = {
  size: 16,
  align: 'left',
  color: 'primary',
  allowFontScaling: false,
};

type Props = {
  children: import('react').ReactNode;
  bold?: boolean;
  italic?: boolean;
  underline?: boolean;
  overrideStyle?: ViewStyle | TextStyle | any;
  flex?: number;
  transform?: textTransForm;
  numberOfLines?: number;
  testID?: string;
  ellipsizeMode?: 'head' | 'middle' | 'tail' | 'clip';
} & typeof defaultProps;

export const isTextStyleKey = (str: string | undefined): str is keyof Styles =>
  str !== undefined;

export const isTextColorKey = (str: string | undefined): str is textColors =>
  str !== undefined;

const Text = (props: Props) => {
  const {theme} = useContext(ThemeContext);

  const {
    size,
    bold,
    italic,
    underline,
    transform,
    align,
    children,
    overrideStyle,
    color,
    flex,
    numberOfLines,
    testID,
    ellipsizeMode,
    allowFontScaling,
  } = props;
  const [styleContainer, setStyleContainer] = useState<ViewStyle>();

  useEffect(() => {
    handleStyleContainer();
  }, [bold, color, theme]);

  const handleStyleContainer = () => {
    let styleContainerArray: TextStyle[] = [];
    const textModifiedStyles = textHelper({
      color,
      flex,
      bold,
      italic,
      underline,
      transform,
      align,
    });
    styleContainerArray = [
      styles.common,
      ...styleContainerArray,
      ...textModifiedStyles,
    ];
    const testColor = textColorHelper(theme, color);
    styleContainerArray.push({color: testColor, fontSize: size});
    if (overrideStyle) {
      styleContainerArray.push(overrideStyle);
    }
    const computedStyle = StyleSheet.flatten(styleContainerArray);
    setStyleContainer(computedStyle);
  };

  return (
    <TextComponent
      ellipsizeMode={ellipsizeMode}
      testID={testID}
      style={styleContainer}
      numberOfLines={numberOfLines}
      allowFontScaling={allowFontScaling}
    >
      {children}
    </TextComponent>
  );
};

Text.defaultProps = defaultProps;

export default memo(Text);
