import React, {useCallback, useEffect, useRef, useState} from 'react';
import {
  DeviceEventEmitter,
  TouchableOpacity,
  StyleSheet,
  View,
} from 'react-native';
import Text from '../Text/Text';
import {
  SEMANTIC_INFO_BLUE_PRIMARY,
  SEMANTIC_RED_PRIMARY,
  SEMANTIC_SUCCESS_GREEN_PRIMARY,
  SEMANTIC_WARNING_ORANGE_PRIMARY,
} from '../UikitUtils/colors';
import {SHOW_TOAST_MESSAGE} from '../UikitUtils/constants';
import {PositionTypes} from './Toast';

export type ToastType = 'info' | 'success' | 'danger' | 'warning';

type colors = {
  info: string;
  success: string;
  danger: string;
  warning: string;
};

const colors: colors = {
  info: SEMANTIC_INFO_BLUE_PRIMARY,
  success: SEMANTIC_SUCCESS_GREEN_PRIMARY,
  danger: SEMANTIC_RED_PRIMARY,
  warning: SEMANTIC_WARNING_ORANGE_PRIMARY,
};

const styles = StyleSheet.create({
  overAll: {
    position: 'absolute',
    left: '4%',
    right: '4%',
    zIndex: 999,
    elevation: 1,
    borderRadius: 4,
  },
  top: {
    top: '4%',
  },
  center: {
    top: '50%',
  },
  bottom: {
    bottom: '4%',
  },
  textStyle: {
    padding: 14,
  },
});

export type Insets = {
  top: number | string;
  bottom: number | string;
};

type Props = {
  insets: Insets;
};

const ToastBar = ({insets}: Props) => {
  const [messageType, setMessageType] = useState<ToastType>('success');
  const [message, setMessage] = useState(null);
  const [isPosition, setPosition] = useState<PositionTypes>('TOP');
  const [timeOutDuration, setTimeOutDuration] = useState(2500);
  const [isHide, setHide] = useState(false);
  const [isAnimationEnd, setAnimationEnd] = useState(false);

  let positionStyle;

  if (isPosition === 'TOP') {
    positionStyle = {top: insets.top};
  } else if (isPosition === 'CENTER') {
    positionStyle = styles.center;
  } else {
    positionStyle = {bottom: insets.bottom};
  }

  const timeOutRef = useRef<any>(null);

  const onNewToast = (data: any) => {
    setAnimationEnd(true);
    setHide(true);
    setMessage(data.message);
    setMessageType(data.type);

    if (data.duration === 'LONG') {
      setTimeOutDuration(2000);
    } else {
      setTimeOutDuration(1000);
    }
    if (data.position) {
      setPosition(data.position);
    }
  };

  const closeToast = useCallback(() => {
    setHide(false);
    setTimeOutDuration(timeOutDuration);
    clearInterval(timeOutRef.current);
  }, []);

  useEffect(() => {
    if (message) {
      timeOutRef.current = setInterval(() => {
        if (timeOutDuration < 0) {
          closeToast();
        } else {
          setTimeOutDuration(prev => prev - 1000);
        }
      }, 1000);
    }

    return () => {
      clearInterval(timeOutRef.current);
    };
  }, [closeToast, message, timeOutDuration]);

  useEffect(() => {
    DeviceEventEmitter.addListener(SHOW_TOAST_MESSAGE, onNewToast);
    return () => {
      DeviceEventEmitter.removeAllListeners();
    };
  }, []);

  if (!isAnimationEnd) {
    return null;
  }

  return isHide ? (
    <View
      style={[
        {
          backgroundColor: colors[messageType],
        },
        styles.overAll,
        positionStyle,
      ]}
    >
      <TouchableOpacity onPress={closeToast}>
        <Text
          size={16}
          align="center"
          color="white"
          overrideStyle={styles.textStyle}
        >
          {message}
        </Text>
      </TouchableOpacity>
    </View>
  ) : (
    <></>
  );
};

ToastBar.defaultProps = {
  insets: {
    top: '4%',
    bottom: '4%',
  },
};
export default ToastBar;
