/* eslint-env browser */
import moment from 'moment';
import {useEffect, createRef, useState, useRef, useCallback} from 'react';
import {Platform} from 'react-native';
import {ThemeType} from './themeTypes';
import {isEmpty} from './validators';

export const getTheme = (): ThemeType => 'app';

export const convertCurrencyFormat = (value: string | number) => {
  if (typeof value === 'number' || typeof value === 'string') {
    return value.toString().replace(/(\d)(?=(\d\d)+\d$)/g, '$1,');
  }
  return '0';
};

export const isValidDate = (date: string): boolean => {
  const timestamp = Date.parse(date);
  return Number.isNaN(timestamp) === false;
};

export const useComponentVisible = (initialIsVisible: boolean) => {
  const [isComponentVisible, setIsComponentVisible] = useState(
    initialIsVisible,
  );
  const ref = createRef<any>();

  const handleClickOutside = (event: {target: any}) => {
    if (ref.current && !ref.current.contains(event.target)) {
      setIsComponentVisible(false);
    }
  };

  useEffect(() => {
    if (typeof Window !== 'undefined') {
      document.addEventListener('click', handleClickOutside, true);
    }
    return () => {
      if (ref && typeof Window !== 'undefined') {
        document.removeEventListener('click', handleClickOutside, true);
      }
    };
  });

  return {ref, isComponentVisible, setIsComponentVisible};
};

export const removeUnderScores = (status: string) => {
  if (typeof status === 'string') {
    return status.replace(/_/g, ' ');
  }
  return '';
};

export const resolveObj = (obj: any, path: string): string => {
  if (typeof path !== 'string' || isEmpty(path)) {
    return '';
  }
  return path
    .split('.')
    .reduce((prev, curr) => (prev && prev[curr] ? prev[curr] : ''), obj);
};

export const openNewWindowHelper = (url: string) => {
  if (typeof Window !== 'undefined') {
    const urlChecker = url[0] === 'h' ? url : window.location.origin + url;
    window.open(urlChecker);
  }
};

export const dateFormatter = (timestamp: string) => {
  if (isValidDate(timestamp)) {
    const formattedTimeStamp = new Date(timestamp).toLocaleString('default', {
      timeZone: 'Asia/Dhaka',
    });

    const monthNames = [
      'Jan',
      'Feb',
      'Mar',
      'Apr',
      'May',
      'Jun',
      'Jul',
      'Aug',
      'Sep',
      'Oct',
      'Nov',
      'Dec',
    ];

    const date = new Date(formattedTimeStamp).getDate();
    const month = new Date(formattedTimeStamp).getMonth();
    const year = new Date(formattedTimeStamp).getFullYear();

    return `${date} ${monthNames[month]} ${year}`;
  }
  return 'Invalid Date';
};

export const useStateCallback = (initialState: any) => {
  const [state, setState] = useState(initialState);
  const callBackRef = useRef<any>(null);

  const setStateCallback = useCallback((updatedState: any, callBack: any) => {
    callBackRef.current = callBack;
    setState(updatedState);
  }, []);

  useEffect(() => {
    if (callBackRef.current) {
      callBackRef.current(state);
      callBackRef.current = null;
    }
  }, [state]);

  return [state, setStateCallback];
};

type GetParamsObject = {[key: string]: any};
type GetParamsPath = string;

export const get = (obj: GetParamsObject, path: GetParamsPath): any =>
  path.split('.').reduce((p, c) => (p && p[c]) || null, obj);

export const errorHandler = (response: any) => {
  throw new Error(response.error.message);
};

export type DateType =
  | 'Do MMM YYYY' // 28th Jul 20222;
  | 'DD-MMM-YYYY' // 28-Jul-20222;
  | 'HH:mm' // 14:10
  | 'HH' //24
  | 'mm' // 00
  | 'DD/MM/YYYY HH:mm:ss' // 2022-12-19 15:37:57
  | 'DD-MM-YYYY'; // 19-03-2022

/**
 * Return the requested date format
 * @param value Date string or object
 * @param format returns the specified date format
 * @param isUnix converts to unix and returns unix formatted date
 * @param convertToLocal converts and returns to local timezone
 */
export const getDateString = (
  value: string | number | Object | undefined,
  format: DateType,
  isUnix?: boolean,
  convertToLocal?: boolean,
) => {
  if (
    (typeof value === 'string' ||
      typeof value === 'number' ||
      typeof value === 'object') &&
    !Array.isArray(value) &&
    !isEmpty(format)
  ) {
    if (isUnix) {
      return moment.unix(Number(value)).format(format);
    } else {
      if (convertToLocal) return moment.parseZone(value).local().format(format);
      return moment.parseZone(value).format(format);
    }
  }
  return '';
};

export const codeToArray = (code?: string): string[] => code?.split('') ?? [];

const majorVersionIOS = parseInt(String(Platform.Version), 10);
export const isAutoFillSupported =
  Platform.OS === 'ios' && majorVersionIOS >= 12;
